/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2003 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Definitions of functions on vectors and matrices
  \author Tony Ottosson

  1.21

  2003/06/17 11:48:36
*/

#ifndef __matfunc_h
#define __matfunc_h

#include "base/vec.h"
#include "base/mat.h"
#include "base/converters.h"
#include "base/scalfunc.h"
#include "base/itassert.h"
#include "base/specmat.h"
#include "base/binary.h"

namespace itpp {

  /*!
    \brief Define of absolute(x)
  */
#define absolut(x) (((x)>0) ? (x) : (-(x)))

  /*! 
    \relates Vec
    \brief Length of vector
  */
  template<class T>
    int length(const Vec<T> &v) { return v.length(); }

  /*! 
    \relates Vec
    \brief Length of vector
  */
  template<class T>
    int size(const Vec<T> &v) { return v.length(); }

  /*! 
    \relates Vec
    \brief Sum of all elements in the vector 
  */
  template<class T>
    T sum(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Sum of square of the elements in a vector
  */
  template<class T>
    T sum_sqr(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Maximum value of vector
  */
  template<class T>
    T max(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Minimum value of vector
  */
  template<class T>
    T min(const Vec<T> &in);

  /*! 
    \relates Vec
    \brief Return the postion of the maximum element in the vector
  */
  template<class T>
    int max_index(const Vec<T> &in);

  /*! 
    \relates Vec
    \brief Return the postion of the minimum element in the vector
  */
  template<class T>
    int min_index(const Vec<T> &in);

  /*! 
    \relates Vec
    \brief The product of all elements in the vector
  */
  template<class T>
    T product(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Vector cross product. Vectors need to be of size 3
  */
  template<class T>
    Vec<T> cross(const Vec<T> &v1, const Vec<T> &v2);

  /*! 
    \relates Vec
    \brief The mean value
  */
  double mean(const vec &v);

  /*! 
    \relates Vec
    \brief The mean value
  */
  complex<double> mean(const cvec &v);

  /*! 
    \relates Vec
    \brief The mean value
  */
  double mean(const svec &v);

  /*! 
    \relates Vec
    \brief The mean value
  */
  double mean(const ivec &v);

  /*! 
    \relates Vec
    \brief The geometric mean value
  */
  template<class T>
    double geometric_mean(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief The median
  */
  template<class T>
    double median(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Calculate the 2-norm: norm(v)=sqrt(sum(abs(v).^2))
  */
  template<class T>
    double norm(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Calculate the 2-norm: norm(v)=sqrt(sum(abs(v).^2))
  */
  template<>
    double norm(const cvec &v);

  /*! 
    \relates Vec
    \brief Calculate the p-norm: norm(v,p)=sum(abs(v).^2)^(1/p)
  */
  template<class T>
    double norm(const Vec<T> &v, int p);

  /*! 
    \relates Vec
    \brief Calculate the p-norm: norm(v,p)=sum(abs(v).^2)^(1/p)
  */
  template<>
    double norm(const cvec &v, int p);

  /*! 
    \relates Vec
    \brief The variance of the elements in the vector. Normalized with N-1 to be unbiased.
  */
  template<class T>
    double variance(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief The variance of the elements in the vector. Normalized with N-1 to be unbiased.
  */
  template<>
    double variance(const cvec &v);
	
  /*!
    \relates Vec
    \brief Calculate the energy: squared 2-norm. energy(v)=sum(v.^2)
  */
  template<class T>
    double energy(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Reverse the input vector
  */
  template<class T>
    Vec<T> reverse(const Vec<T> &in);

  /*! 
    \relates Vec
    \brief Repeat each element in the vector norepeats times in sequence
  */
  template<class T>
    Vec<T> repeat(const Vec<T> &v, int norepeats);

  /*! 
    \relates Vec
    \brief Apply arbitrary function to a vector
  */
  template<class T, class fT>
    Vec<T> apply_function(fT (*f)(fT), const Vec<T> &data);

  /*! 
    \relates Vec
    \brief Sort the the vector in increasing order
  */
  template<class T>
    void sort(Vec<T> &data);

  /*! 
    \relates Vec
    \brief Return an index vector corresponding to a sorted vector (increasing order)
  */
  template<class T>
    ivec sort_index(const Vec<T> &data);

  /*! 
    \relates Vec
    \brief Zero-pad a vector to size n
  */
  template<class T>
    Vec<T> zero_pad(const Vec<T> &v, int n);

  /*! 
    \relates Vec
    \brief Zero-pad a vector to the nearest greater power of two
  */
  template<class T>
    Vec<T> zero_pad(const Vec<T> &v);

  /*! 
    \relates Vec
    \brief Zero-pad a matrix to size rows x cols
  */
  template<class T>
    Mat<T> zero_pad(const Mat<T> &v, int rows, int cols);

  //---------- Template functions for Mat<T> ----------------------

  /*! 
    \relates Mat
    \brief Sum of all elements in the matrix 
  */
  template<class T>
    T sum(const Mat<T> &m);

  /*! 
    \relates Mat
    \brief Sum of square of the elements in the matrix
  */
  template<class T>
    T sum_sqr(const Mat<T> &m);

  /*! 
    \relates Mat
    \brief Maximum value of matrix
  */
  template<class T>
    T max(const Mat<T> &m);

  /*! 
    \relates Mat
    \brief Minimum value of matrix
  */
  template<class T>
    T min(const Mat<T> &m);

  /*! 
    \relates Mat
    \brief The index of the element of maximum value
  */
  template<class T>
    void max_index(const Mat<T> &in, int &row, int &col);

  /*! 
    \relates Mat
    \brief The index of the element of minimum value
  */
  template<class T>
    void min_index(const Mat<T> &in, int &row, int &col);

  /*! 
    \relates Mat
    \brief The product of all elements in the vector
  */
  template<class T>
    T product(const Mat<T> &m);

  /*! 
    \relates Mat
    \brief The mean value
  */
  double mean(const mat &m);

  /*! 
    \relates Mat
    \brief The mean value
  */
  complex<double> mean(const cmat &m);

  /*! 
    \relates Mat
    \brief The mean value
  */
  double mean(const smat &m);

  /*! 
    \relates Mat
    \brief The mean value
  */
  double mean(const imat &m);

  /*! 
    \relates Mat
    \brief Returns a diagonal matrix whith the elements of the vector \c in on the diagonal and zeros elsewhere.

    The size of the return matrix will be \f$n \times n\f$, where \f$n\f$ is the length of the input vector \c in.
  */
  template<class T>
    Mat<T> diag(const Vec<T> &in);

  /*! 
    \relates Mat
    \brief Returns in the output wariable \c m a diagonal matrix whith the elements of the vector \c in on the diagonal and zeros elsewhere.
  
    The size of the output matrix \c m will be \f$n \times n\f$, where \f$n\f$ is the length of the input vector \c in.
  */
  template<class T>
    void diag(const Vec<T> &in, Mat<T> &m);

  /*! 
    \relates Mat
    \brief Returns the diagonal elements of the input matrix \c in.

    The input matrix \c m must be a square \f$n \times n\f$ matrix. The size of the output vector will be \f$n\f$.
  */
  template<class T>
    Vec<T> diag(const Mat<T> &in);

  // 
  /*!
    \relates Mat
    \brief Returns a matrix with the elements of the input vector \c main on the diagonal and the elements of the input vector \c sup on the diagonal row above.

    If the number of elements in the vector \c main is \f$n\f$, then the number of elements in the input vector 
    \c sup must be \f$n-1\f$. The size of the return matrix will be \f$n \times n\f$.
  */
  template<class T>
    Mat<T> bidiag(const Vec<T> &main, const Vec<T> &sup);

  /*!
    \relates Mat
    \brief Returns in the output variable \c m a matrix with the elements of the input vector \c main on the diagonal and the elements of the input vector \c sup on the diagonal row above.

    If the number of elements in the vector \c main is \f$n\f$, then the number of elements in the input vector 
    \c sup must be \f$n-1\f$. The size of the output matrix \c m will be \f$n \times n\f$.
  */
  template<class T>
    void bidiag(const Vec<T> &main, const Vec<T> &sup, Mat<T> &m);

  /*!
    \relates Mat
    \brief Returns the main diagonal and the diagonal row above in the two output vectors \c main and \c sup.

    The input matrix \c in must be a square \f$n \times n\f$ matrix. The length of the output vector \c main will be \f$n\f$ 
    and the length of the output vector \c sup will be \f$n-1\f$.
  */
  template<class T>
    void bidiag(const Mat<T> &in, Vec<T> &main, Vec<T> &sup);

  /*!
    \relates Mat
    \brief Returns a matrix with the elements of \c main on the diagonal, the elements of \c sup on the diagonal row above, and the elements of \c sub on the diagonal row below.

    If the length of the input vector \c main is \f$n\f$ then the lengths of the vectors \c sup and \c sub 
    must equal \f$n-1\f$. The size of the return matrix will be \f$n \times n\f$.
  */
  template<class T>
    Mat<T> tridiag(const Vec<T> &main, const Vec<T> &sup, const Vec<T> &sub);

  /*!
    \relates Mat
    \brief Returns in the output matrix \c m a matrix with the elements of \c main on the diagonal, the elements of \c sup on the diagonal row above, and the elements of \c sub on the diagonal row below.
  
    If the length of the input vector \c main is \f$n\f$ then the lengths of the vectors \c sup and \c sub 
    must equal \f$n-1\f$. The size of the output matrix \c m will be \f$n \times n\f$.
  */
  template<class T>
    void tridiag(const Vec<T> &main, const Vec<T> &sup, const Vec<T> &sub, Mat<T> &m);

  /*!
    \relates Mat
    \brief Returns the main diagonal, the diagonal row above, and the diagonal row below int the output vectors \c main, \c sup, and \c sub.

    The input matrix \c in must be a square \f$n \times n\f$ matrix. The length of the output vector \c main will be \f$n\f$ 
    and the length of the output vectors \c sup and \c sup will be \f$n-1\f$.
  */
  template<class T>
    void tridiag(const Mat<T> &in, Vec<T> &main, Vec<T> &sup, Vec<T> &sub);

  /*! 
    \relates Mat
    \brief The trace of the matrix, i.e. the sum of the diagonal elements.
  */
  template<class T>
    T trace(const Mat<T> &in);

  /*! 
    \relates Mat
    \brief Transposition of the matrix returning the transposed matrix in out
  */
  template<class T>
    void transpose(const Mat<T> &m, Mat<T> &out);

  /*! 
    \relates Mat
    \brief Transposition of the matrix
  */
  template<class T>
    Mat<T> transpose(const Mat<T> &m);

  /*! 
    \relates Mat
    \brief Repeats each column norepeats times in sequence
  */
  template<class T>
    Mat<T> repeat(const Mat<T> &m, int norepeats);

  /*! 
    \relates Mat
    \brief Apply arbitrary functions to a matrix
  */
  template<class T, class fT>
    Mat<T> apply_function(fT (*f)(fT), const Mat<T> &data);

  /*!
    \relates Mat
    \brief Row vectorize the matrix [(0,0) (0,1) ... (N-1,N-2) (N-1,N-1)]
  */
  template<class T>
    Vec<T> rvectorize(const Mat<T> &m);

  /*!
    \relates Mat
    \brief Column vectorize the matrix [(0,0) (1,0) ... (N-2,N-1) (N-1,N-1)]
  */
  template<class T>
    Vec<T> cvectorize(const Mat<T> &m);

  /*!
    \relates Mat
    \brief Reshape the matrix into an rows*cols matrix

    The data is taken columnwise from the original matrix and written columnwise into the new matrix.
  */
  template<class T>
    Mat<T> reshape(const Mat<T> &m, int rows, int cols);

  /*!
    \relates Mat
    \brief Reshape the vector into an rows*cols matrix

    The data is element by element from the vector and written columnwise into the new matrix.
  */
  template<class T>
    Mat<T> reshape(const Vec<T> &m, int rows, int cols);

  /*! 
    \relates Vec
    \brief Upsample a vector by incerting \a (usf-1) zeros after each sample
  */ 
  template<class T> 
    void upsample(const Vec<T> &v, int usf, Vec<T> &u);

  /*! 
    \relates Vec
    \brief Upsample a vector by incerting \a (usf-1) zeros after each sample
  */
  template<class T>
    Vec<T> upsample(const Vec<T> &v, int usf);

  /*! 
    \relates Mat
    \brief Upsample each column by incerting \a (usf-1) zeros after each column
  */

  template<class T>
    void upsample(const Mat<T> &v, int usf, Mat<T> &u);

  /*! 
    \relates Mat
    \brief Upsample each column by incerting \a (usf-1) zeros after each column
  */
  template<class T>
    Mat<T> upsample(const Mat<T> &v, int upsamplefactor);

  /*!
    \relates Mat
    \brief Upsample each column by a factor of  \a (usf-1) by linear interpolation
  */
  template<class T>
    void lininterp(const Mat<T> &v, int usf, Mat<T> &u);

  /*!
    \relates Mat
    \brief Upsample each column by a factor of  \a (usf-1) by linear interpolation
  */
  template<class T>
    Mat<T> lininterp(const Mat<T> &v, int usf);

  /*!
    \relates Mat
    \brief Upsample by a factor of  \a (usf-1) by linear interpolation
  */
  template<class T>
    void lininterp(const Vec<T> &v, int usf, Vec<T> &u);

  /*!
    \relates Mat
    \brief Upsample by a factor of  \a (usf-1) by linear interpolation
  */
  template<class T>
    Vec<T> lininterp(const Vec<T> &v, int usf);

  //----------------------------------------------------------------------------
  // Implementation of templated functions start here
  //----------------------------------------------------------------------------

  //---------- Template functions for Vec<T> ----------------------

  // sum seems to be defined elsewhere. A change to sum2 everywhere gets rid of the doxygen warning.
  template<class T>
    T sum(const Vec<T> &v)
    {
      T M=0;

      for (int i=0;i<v.length();i++)
	M+=v[i];

      return M;
    }

  template<class T>
    T sum_sqr(const Vec<T> &v)
    {
      T M=0;

      for (int i=0; i<v.length(); i++)
	M += v[i] * v[i];

      return M;
    }

  template<class T>
    T max(const Vec<T> &in)
    {
      T	maxdata=in[0];
      for (int i=1;i<in.length();i++)
	if (in[i]>maxdata)
	  maxdata=in[i];
      return maxdata;
    }

  template<class T>
    T min(const Vec<T> &in)
    {
      T mindata=in[0];
      for (int i=1;i<in.length();i++)
	if (in[i]<mindata)
	  mindata=in[i];
      return mindata;
    }

  template<class T>
    int max_index(const Vec<T> &in)
    {
      int	maxindex=0;
      for (int i=0;i<in.length();i++)
	if (in[i]>in[maxindex])
	  maxindex=i;
      return maxindex;
    }

  template<class T>
    int min_index(const Vec<T> &in)
    {
      int	minindex=0;
      for (int i=0;i<in.length();i++)
	if (in[i]<in[minindex])
	  minindex=i;
      return minindex;
    }

  template<class T>
    T product(const Vec<T> &v)
    {
      double lnM=0;
      for (int i=0;i<v.length();i++)
	lnM += log(static_cast<double>(v[i]));
      return static_cast<T>(exp(lnM));
    }

  template<class T>
    Vec<T> cross(const Vec<T> &v1, const Vec<T> &v2)
    {
      it_assert( v1.size() == 3 && v2.size() == 3, "cross: vectors should be of size 3");

      Vec<T> r(3);

      r(0) = v1(1) * v2(2) - v1(2) * v2(1);
      r(1) = v1(2) * v2(0) - v1(0) * v2(2);
      r(2) = v1(0) * v2(1) - v1(1) * v2(0);

      return r;
    }

  template<class T>
    double geometric_mean(const Vec<T> &v)
    {
      return exp(log(static_cast<double>(product(v)))/v.length());
    }

  template<class T>
    double median(const Vec<T> &v)
    {
      Vec<T> invect(v);
      sort(invect);
      return (double)(invect[(invect.length()-1)/2]+invect[invect.length()/2])/2.0;
    }

  /*!
    \brief Template specialization of \c norm for \c cvec
  */
  template<>
    double norm(const cvec &v);

  template<class T>
    double norm(const Vec<T> &v)
    {
      int i;
      double E=0.0;

      for (i=0; i<v.length(); i++)
	E += sqr( static_cast<double>(v[i]) );

      return std::sqrt(E);
    }

  /*!
    \brief Template specialization of \c norm for \c cvec
  */
  template<>
    double norm(const cvec &v, int p);

  template<class T>
    double norm(const Vec<T> &v, int p)
    {
      int i;
      double E=0;

      for (i=0;i<v.length();i++)
	E+=std::pow(fabs(v[i]),(double)p); // fabs() shoud be OK

      return std::pow(E,1.0/(double)p);
    }

  /*!
    \brief Template specialization of \c variance for \c cvec 
  */
  template<>
    double variance(const cvec &v);

  template<class T>
    double variance(const Vec<T> &v)
    {
      int len = v.length();
      const T *p=v._data();
      double sum=0.0, sq_sum=0.0;

      for (int i=0; i<len; i++, p++) {
	sum += *p;
	sq_sum += *p * *p;
      }

      return (double)(sq_sum - sum*sum/len) / (len-1);
    }

  template<class T>
    double energy(const Vec<T> &v)
    {
      return sqr(norm(v));
    }

  template<class T>
    Vec<T> reverse(const Vec<T> &in)
    {
      int i, s=in.length();

      Vec<T> out(s);
      for (i=0;i<s;i++)
	out[i]=in[s-1-i];
      return out;
    }

  template<class T>
    Vec<T> repeat(const Vec<T> &v, int norepeats)
    {
      Vec<T> temp(v.length()*norepeats);

      for(int i=0;i<v.length();i++) {
	for(int j=0;j<norepeats;j++)
	  temp(i*norepeats+j)=v(i);
      }
      return temp;
    }

  template<class T, class fT>
    Vec<T> apply_function(fT (*f)(fT), const Vec<T> &data)
    {
      Vec<T> out(data.length());

      for (int i=0;i<data.length();i++)
	out[i]=static_cast<T>(f(static_cast<fT>(data[i])));
      return out;
    }

  template<class T> void QS(int low, int high, Vec<T> &data) {
    int plow, phigh;
    T a,test;

    if (high>low) {
      a=data[low];
      plow=low;
      phigh=high;
      test=data[phigh];
      while (plow<phigh) {
	if (test<a) {
	  data[plow]=test;
	  plow++;
	  test=data[plow];
	} else {
	  data[phigh]=test;
	  phigh--;
	  test=data[phigh];
	}
      }
      data[plow]=a;
      QS(low,plow-1,data);
      QS(plow+1,high,data);
    }
  }

  template<class T>
    void sort(Vec<T> &data)
    {
      QS(0,data.size()-1,data);
    }

  template<class T>
    void QSindex(int low, int high, ivec &indexlist, const Vec<T> &data)
    {
      int plow,phigh,testindex,aindex;
      T a,test;

      if (high>low) {
	aindex=indexlist[low];
	a=data[aindex];
	plow=low;
	phigh=high;
	testindex=indexlist[phigh];
	test=data[testindex];
	while (plow<phigh) {
	  if (test<a) {
	    indexlist[plow]=testindex;
	    plow++;
	    testindex=indexlist[plow];
	    test=data[testindex];
	  } else {
	    indexlist[phigh]=testindex;
	    phigh--;
	    testindex=indexlist[phigh];
	    test=data[testindex];
	  }
	}
	indexlist[plow]=aindex;
	QSindex(low,plow-1,indexlist,data);
	QSindex(plow+1,high,indexlist,data);
      }
    }

  template<class T>
    ivec sort_index(const Vec<T> &data)
    {
      int N=data.length(),i;
      ivec indexlist(N);

      for(i=0;i<N;i++) {
	indexlist(i)=i;
      }
      QSindex(0,N-1,indexlist,data);
      return indexlist;
    }

  template<class T>
    Vec<T> zero_pad(const Vec<T> &v, int n)
    {
      it_assert(n>=v.size(), "zero_pad() cannot shrink the vector!");
      Vec<T> v2(n);
      v2.set_subvector(0, v.size()-1, v);
      if (n > v.size())
	v2.set_subvector(v.size(), n-1, T(0));

      return v2;
    }

  template<class T>
    Vec<T> zero_pad(const Vec<T> &v)
    {
      int n = pow2(needed_bits(v.size()));

      return n==v.size() ? v : zero_pad(v, n);
    }

  template<class T>
    Mat<T> zero_pad(const Mat<T> &m, int rows, int cols)
    {
      it_assert(rows>=m.rows() && cols>=m.cols(), "zero_pad() cannot shrink the matrix!");
      Mat<T> m2(rows, cols);
      m2.set_submatrix(0,m.rows()-1,0,m.cols()-1, m);
      if (cols > m.cols()) // Zero
	m2.set_submatrix(0,m.rows()-1, m.cols(),cols-1, T(0));
      if (rows > m.rows()) // Zero
	m2.set_submatrix(m.rows(), rows-1, 0, cols-1, T(0));

      return m2;
    }
  //---------- Template functions for Mat<T> ----------------------

  template<class T>
    T sum(const Mat<T> &v)
    {
      int i, j;
      T M=0;

      for (i=0; i<v.rows(); i++)
	for (j=0; j<v.cols(); j++)
	  M += v(i,j);

      return M;
    }

  template<class T>
    T sum_sqr(const Mat<T> &v)
    {
      int i, j;
      T M=0;

      for (i=0; i<v.rows(); i++)
	for (j=0; j<v.cols(); j++)
	  M += v(i,j) * v(i,j);

      return M;
    }

  template<class T>
    T max(const Mat<T> &m)
    {
      T maxdata = m(0,0);
      int i, j;

      for (i=0; i<m.rows(); i++)
	for (j=0; j<m.cols(); j++)
	  if (m(i,j) > maxdata)
	    maxdata = m(i,j);

      return maxdata;
    }

  template<class T>
    T min(const Mat<T> &m)
    {
      T mindata = m(0,0);
      int i, j;

      for (i=0; i<m.rows(); i++)
	for (j=0; j<m.cols(); j++)
	  if (m(i,j) < mindata)
	    mindata = m(i,j);

      return mindata;
    }

  template<class T>
    void max_index(const Mat<T> &m, int &row, int &col)
    {
      T maxdata = m(0,0);
      int i, j;

      row = col = 0;
      for (i=0; i<m.rows(); i++)
	for (j=0; j<m.cols(); j++)
	  if (m(i,j) > maxdata) {
	    row = i;
	    col = j;
	    maxdata = m(i,j);
	  }
    }

  template<class T>
    void min_index(const Mat<T> &m, int &row, int &col)
    {
      T mindata = m(0,0);
      int i, j;

      row = col = 0;
      for (i=0; i<m.rows(); i++)
	for (j=0; j<m.cols(); j++)
	  if (m(i,j) < mindata) {
	    row = i;
	    col = j;
	    mindata = m(i,j);
	  }
    }

  template<class T>
    T product(const Mat<T> &m)
    {
      int i, j;
      double lnM=0;

      for (i=0; i<m.rows(); i++)
	for (j=0; j<m.cols(); j++)
	  lnM += log(m(i,j));

      return static_cast<T>(exp(lnM));
    }

  template<class T>
    Mat<T> diag(const Vec<T> &in)
    {
      Mat<T> m(in.size(), in.size());
      m = T(0);
      for (int i=in.size()-1; i>=0; i--)
	m(i,i) = in(i);
      return m;
    }

  template<class T>
    void diag(const Vec<T> &in, Mat<T> &m)
    {
      m.set_size(in.size(), in.size(), false);
      m = T(0);
      for (int i=in.size()-1; i>=0; i--)
	m(i,i) = in(i);
    }

  template<class T>
    Vec<T> diag(const Mat<T> &in)
    {
      Vec<T> t(std::min(in.rows(), in.cols()));

      for (int i=0; i<t.size(); i++)
	t(i) = in(i,i);

      return t;
    }

  template<class T>
    Mat<T> bidiag(const Vec<T> &main, const Vec<T> &sup)
    {
      it_assert(main.size() == sup.size()+1, "bidiag()");

      int n=main.size();
      Mat<T> m(n, n);
      m = T(0);
      for (int i=0; i<n-1; i++) {
	m(i,i) = main(i);
	m(i,i+1) = sup(i);
      }
      m(n-1,n-1) = main(n-1);

      return m;
    }

  template<class T>
    void   bidiag(const Vec<T> &main, const Vec<T> &sup, Mat<T> &m)
    {
      it_assert(main.size() == sup.size()+1, "bidiag()");

      int n=main.size();
      m.set_size(n, n);
      m = T(0);
      for (int i=0; i<n-1; i++) {
	m(i,i) = main(i);
	m(i,i+1) = sup(i);
      }
      m(n-1,n-1) = main(n-1);
    }

  template<class T>
    void bidiag(const Mat<T> &m, Vec<T> &main, Vec<T> &sup)
    {
      it_assert(m.rows() == m.cols(), "bidiag(): Matrix must be square!");

      int n=m.cols();
      main.set_size(n);
      sup.set_size(n-1);
      for (int i=0; i<n-1; i++) {
	main(i) = m(i,i);
	sup(i) = m(i,i+1);
      }
      main(n-1) = m(n-1,n-1);
    }

  template<class T>
    Mat<T> tridiag(const Vec<T> &main, const Vec<T> &sup, const Vec<T> &sub)
    {
      it_assert(main.size()==sup.size()+1 && main.size()==sub.size()+1, "bidiag()");

      int n=main.size();
      Mat<T> m(n, n);
      m = T(0);
      for (int i=0; i<n-1; i++) {
	m(i,i) = main(i);
	m(i,i+1) = sup(i);
	m(i+1,i) = sub(i);
      }
      m(n-1,n-1) = main(n-1);

      return m;
    }

  template<class T>
    void tridiag(const Vec<T> &main, const Vec<T> &sup, const Vec<T> &sub, Mat<T> &m)
    {
      it_assert(main.size()==sup.size()+1 && main.size()==sub.size()+1, "bidiag()");

      int n=main.size();
      m.set_size(n, n);
      m = T(0);
      for (int i=0; i<n-1; i++) {
	m(i,i) = main(i);
	m(i,i+1) = sup(i);
	m(i+1,i) = sub(i);
      }
      m(n-1,n-1) = main(n-1);
    }

  template<class T>
    void tridiag(const Mat<T> &m, Vec<T> &main, Vec<T> &sup, Vec<T> &sub)
    {
      it_assert(m.rows() == m.cols(), "tridiag(): Matrix must be square!");

      int n=m.cols();
      main.set_size(n);
      sup.set_size(n-1);
      sub.set_size(n-1);
      for (int i=0; i<n-1; i++) {
	main(i) = m(i,i);
	sup(i) = m(i,i+1);
	sub(i) = m(i+1,i);
      }
      main(n-1) = m(n-1,n-1);
    }

  template<class T>
    T trace(const Mat<T> &in)
    {
      return sum(diag(in));
    }

  template<class T>
    void transpose(const Mat<T> &m, Mat<T> &out)
    {
      out.set_size(m.cols(),m.rows(), false);

      for (int i=0; i<m.rows(); i++) {
	for (int j=0; j<m.cols(); j++) {
	  out(j,i) = m(i,j);
	}
      }
    }

  template<class T>
    Mat<T> transpose(const Mat<T> &m)
    {
      Mat<T> temp(m.cols(),m.rows());

      for (int i=0; i<m.rows(); i++) {
	for (int j=0; j<m.cols(); j++) {
	  temp(j,i) = m(i,j);
	}
      }
      return temp;
    }

  template<class T>
    Mat<T> repeat(const Mat<T> &v, int norepeats)
    {
      Mat<T> temp(v.rows(),v.cols()*norepeats);
      for (int j=0;j<v.cols();j++) {
	for (int i=0;i<norepeats;i++) {
	  temp.set_col(j*norepeats+i,v.get_col(j));
	}
      }
      return temp;
    }

  template<class T, class fT>
    Mat<T> apply_function(fT (*f)(fT), const Mat<T> &data)
    {
      Mat<T> out(data.rows(),data.cols());

      for (int i=0;i<out.rows();i++)
	for (int j=0;j<out.cols();j++)
	  out(i,j)=static_cast<T>(f(static_cast<fT>(data(i,j))));

      return out;
    }

  template<class T>
    Vec<T> rvectorize(const Mat<T> &m)
    {
      int i, j, n=0, r=m.rows(), c=m.cols();
      Vec<T> v(r * c);

      for (i=0; i<r; i++)
	for (j=0; j<c; j++)
	  v(n++) = m(i,j);

      return v;
    }

  template<class T>
    Vec<T> cvectorize(const Mat<T> &m)
    {
      int i, j, n=0, r=m.rows(), c=m.cols();
      Vec<T> v(r * c);

      for (j=0; j<c; j++)
	for (i=0; i<r; i++)
	  v(n++) = m(i,j);

      return v;
    }

  template<class T>
    Mat<T> reshape(const Mat<T> &m, int rows, int cols)
    {
      it_assert1(m.rows()*m.cols() == rows*cols, "Mat<T>::reshape: Sizes must match");
      Mat<T> temp(rows, cols);
      int i, j, ii=0, jj=0;
      for (j=0; j<m.cols(); j++) {
	for (i=0; i<m.rows(); i++) {
	  temp(ii++,jj) = m(i,j);
	  if (ii == rows) {
	    jj++; ii=0;
	  }
	}
      }
      return temp;
    }

  template<class T>
    Mat<T> reshape(const Vec<T> &v, int rows, int cols)
    {
      it_assert1(v.size() == rows*cols, "Mat<T>::reshape: Sizes must match");
      Mat<T> temp(rows, cols);
      int i, j, ii=0;
      for (j=0; j<cols; j++) {
	for (i=0; i<rows; i++) {
	  temp(i,j) = v(ii++);
	}
      }
      return temp;
    }

  template<class T>
    void upsample(const Vec<T> &v, int usf, Vec<T> &u)
    {
      it_assert1(usf >= 1, "upsample: upsampling factor must be equal or greater than one" );
      u.set_size(v.length()*usf);
      u.clear();
      for(long i=0;i<v.length();i++)
	u(i*usf)=v(i);
    }

  template<class T>
    Vec<T> upsample(const Vec<T> &v, int usf)
    {
      Vec<T> u;
      upsample(v,usf,u);
      return u;
    }

  template<class T>
    void upsample(const Mat<T> &v, int usf, Mat<T> &u)
    {
      it_assert1(usf >= 1, "upsample: upsampling factor must be equal or greater than one" );
      u.set_size(v.rows(),v.cols()*usf);
      u.clear();
      for (long j=0;j<v.cols();j++)
	u.set_col(j*usf,v.get_col(j));
    }

  template<class T>
    Mat<T> upsample(const Mat<T> &v, int usf)
    {
      Mat<T> u;
      upsample(v,usf,u);
      return u;
    }

  template<class T>
    void lininterp(const Mat<T> &v, int usf, Mat<T> &u)
    {
      it_assert1(usf >= 1, "lininterp: upsampling factor must be equal or greater than one" );
      long L = (v.cols()-1)*usf+1;
      u.set_size(v.rows(),L);
      for (long i = 0; i < v.rows(); i++){
	for (long j = 0; j < L-1; j++)
	  u(i,j) = (v(i,j/usf) + (j % usf)/((double)usf)*(v(i,(j+usf)/usf)-v(i,j/usf)));
	u(i,L-1) = v(i,v.cols()-1);
      }
    }

  template<class T>
    Mat<T> lininterp(const Mat<T> &v, int usf)
    {
      Mat<T> u;
      lininterp(v,usf,u);
      return u;
    }

  template<class T>
    void lininterp(const Vec<T> &v, int usf, Vec<T> &u)
    {
      it_assert1(usf >= 1, "lininterp: upsampling factor must be equal or greater than one" );
      long L = (v.length()-1)*usf+1;
      u.set_size(L);
      for (long j = 0; j < L-1; j++) {
	//u(j) = (v(j/usf) + (j % usf)/((float)usf)*(v((j+usf)/usf)-v(j/usf)));
	u(j) = (v(j/usf) + (j % usf)/((double)usf)*(v((j+usf)/usf)-v(j/usf)));
      }
      u(L-1) = v(v.length()-1);
    }

  template<class T>
    Vec<T> lininterp(const Vec<T> &v, int usf)
    {
      Vec<T> u;
      lininterp(v,usf,u);
      return u;
    }

} //namespace itpp

#endif // __matfunc_h

